/*
** Module   : ToggleBoxImpl.java
** Abstract :
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20140610 Initial implementation.
*/

package com.goldencode.p2j.ui.chui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * Specific implementation of a "toggle box" with a text label for CHUI.
 */
@SuppressWarnings("unchecked")
public class ToggleBoxImpl
extends ToggleBox
{
   /**
    * This constructor creates a toggle-box with a given text label.
    * 
    * @param   owner
    *          The parent container, frame or dialog which will include this toggle-box.
    * @param   text
    *          The string to be used as the toggle box text label.
    */
   ToggleBoxImpl(Container owner, String text) 
   {
      super(owner, text);
   }

   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   cfg
    *          Configuration data.
    */
   public ToggleBoxImpl(ComponentConfig cfg)
   {
      super(cfg);
   }
   
   /**
    * Draw the button. Called by this ToggleBox's parent container.
    */
   @Override
   public void draw()
   {
      // get the absolute origin of this component
      Point origin = screenLocation();

      screen().at(origin);
      Color dc = UiUtils.getDcolor(this);
      Color pf = config().getPfcolor();

      // obtain the full text of the toggle-box's label
      String label = textLabel();
      
      // calculate the total size of the toggle-box
      int tbTotalSize = width();
      
      // calculate the max size of the text part, 3 cells should be reserved for check-box
      int textMaxSize = tbTotalSize - 3;
      
      // calculate the actual text length (may be shorter or longer than the text size)
      int textActualSize = label.length(); 
      
      StringBuffer sb = new StringBuffer();
      
      // there is at least always the check-box with or without selection
      drawCheckBox(isEnabled() && widgetFocused, pf, dc);

      if (textMaxSize > 0)
      {
         // the full text labes size is bigger than space reserved for complete
         // toggle-box rendering
         if (textMaxSize < textActualSize)
         {
            // right truncate the text
            sb.append(label.substring(0, textMaxSize));
         }
         else
         {
            // we are just big enough to hold the full text
            sb.append(label);
         }
      }
      
      // actually draw the text label for check-box
      screen().append(sb.toString(), dc);
   }

   /**
    * Draws check box itself ("[X]" or "[ ]").
    *
    * @param highlighted
    *        Indicates whether the button is highlighted.
    * @param pf
    *        The color used inside this check box under focus.
    * @param dc
    *        The color which corresponds DCOLOR attribute for this radio
    *        button.
    */
   private void drawCheckBox(boolean highlighted, Color pf, Color dc)
   {
      screen().append("[", highlighted ? pf : dc);
      screen().append(config().isChecked() ? "X" : " ", highlighted ? pf : dc);
      screen().append("]", highlighted ? pf : dc);
   }
}
